"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tokens_public_1 = require("./tokens_public");
var lexer_1 = require("./lexer");
var utils_1 = require("../utils/utils");
var tokens_1 = require("./tokens");
var LexerDefinitionErrorType;
(function (LexerDefinitionErrorType) {
    LexerDefinitionErrorType[LexerDefinitionErrorType["MISSING_PATTERN"] = 0] = "MISSING_PATTERN";
    LexerDefinitionErrorType[LexerDefinitionErrorType["INVALID_PATTERN"] = 1] = "INVALID_PATTERN";
    LexerDefinitionErrorType[LexerDefinitionErrorType["EOI_ANCHOR_FOUND"] = 2] = "EOI_ANCHOR_FOUND";
    LexerDefinitionErrorType[LexerDefinitionErrorType["UNSUPPORTED_FLAGS_FOUND"] = 3] = "UNSUPPORTED_FLAGS_FOUND";
    LexerDefinitionErrorType[LexerDefinitionErrorType["DUPLICATE_PATTERNS_FOUND"] = 4] = "DUPLICATE_PATTERNS_FOUND";
    LexerDefinitionErrorType[LexerDefinitionErrorType["INVALID_GROUP_TYPE_FOUND"] = 5] = "INVALID_GROUP_TYPE_FOUND";
    LexerDefinitionErrorType[LexerDefinitionErrorType["PUSH_MODE_DOES_NOT_EXIST"] = 6] = "PUSH_MODE_DOES_NOT_EXIST";
    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE"] = 7] = "MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE";
    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY"] = 8] = "MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY";
    LexerDefinitionErrorType[LexerDefinitionErrorType["MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST"] = 9] = "MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST";
    LexerDefinitionErrorType[LexerDefinitionErrorType["LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED"] = 10] = "LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED";
    LexerDefinitionErrorType[LexerDefinitionErrorType["SOI_ANCHOR_FOUND"] = 11] = "SOI_ANCHOR_FOUND";
})(LexerDefinitionErrorType = exports.LexerDefinitionErrorType || (exports.LexerDefinitionErrorType = {}));
var DEFAULT_LEXER_CONFIG = {
    deferDefinitionErrorsHandling: false,
    positionTracking: "full",
    debug: false
};
Object.freeze(DEFAULT_LEXER_CONFIG);
var Lexer = (function () {
    /**
     * @param {SingleModeLexerDefinition | IMultiModeLexerDefinition} lexerDefinition -
     *  Structure composed of constructor functions for the Tokens types this lexer will support.
     *
     *  In the case of {SingleModeLexerDefinition} the structure is simply an array of Token constructors.
     *  In the case of {IMultiModeLexerDefinition} the structure is an object with two properties:
     *    1. a "modes" property where each value is an array of Token.
     *    2. a "defaultMode" property specifying the initial lexer mode.
     *
     *  constructors.
     *
     *  for example:
     *  {
     *     "modes" : {
     *     "modeX" : [Token1, Token2]
     *     "modeY" : [Token3, Token4]
     *     }
     *
     *     "defaultMode" : "modeY"
     *  }
     *
     *  A lexer with {MultiModesDefinition} is simply multiple Lexers where only one (mode) can be active at the same time.
     *  This is useful for lexing languages where there are different lexing rules depending on context.
     *
     *  The current lexing mode is selected via a "mode stack".
     *  The last (peek) value in the stack will be the current mode of the lexer.
     *
     *  Each Token class can define that it will cause the Lexer to (after consuming an instance of the Token):
     *  1. PUSH_MODE : push a new mode to the "mode stack"
     *  2. POP_MODE  : pop the last mode from the "mode stack"
     *
     *  Examples:
     *       export class Attribute extends Token {
     *          static PATTERN = ...
     *          static PUSH_MODE = "modeY"
     *       }
     *
     *       export class EndAttribute extends Token {
     *          static PATTERN = ...
     *          static POP_MODE = true
     *       }
     *
     *  The Token constructors must be in one of these forms:
     *
     *  1. With a PATTERN property that has a RegExp value for tokens to match:
     *     example: -->class Integer extends Token { static PATTERN = /[1-9]\d }<--
     *
     *  2. With a PATTERN property that has the value of the var Lexer.NA defined above.
     *     This is a convenience form used to avoid matching Token classes that only act as categories.
     *     example: -->class Keyword extends Token { static PATTERN = NA }<--
     *
     *
     *   The following RegExp patterns are not supported:
     *   a. '$' for match at end of input
     *   b. /b global flag
     *   c. /m multi-line flag
     *
     *   The Lexer will identify the first pattern that matches, Therefor the order of Token Constructors may be significant.
     *   For example when one pattern may match a prefix of another pattern.
     *
     *   Note that there are situations in which we may wish to order the longer pattern after the shorter one.
     *   For example: keywords vs Identifiers.
     *   'do'(/do/) and 'donald'(/w+)
     *
     *   * If the Identifier pattern appears before the 'do' pattern, both 'do' and 'donald'
     *     will be lexed as an Identifier.
     *
     *   * If the 'do' pattern appears before the Identifier pattern 'do' will be lexed correctly as a keyword.
     *     however 'donald' will be lexed as TWO separate tokens: keyword 'do' and identifier 'nald'.
     *
     *   To resolve this problem, add a static property on the keyword's constructor named: LONGER_ALT
     *   example:
     *
     *       export class Identifier extends Keyword { static PATTERN = /[_a-zA-Z][_a-zA-Z0-9]/ }
     *       export class Keyword extends Token {
     *          static PATTERN = lex.NA
     *          static LONGER_ALT = Identifier
     *       }
     *       export class Do extends Keyword { static PATTERN = /do/ }
     *       export class While extends Keyword { static PATTERN = /while/ }
     *       export class Return extends Keyword { static PATTERN = /return/ }
     *
     *   The lexer will then also attempt to match a (longer) Identifier each time a keyword is matched.
     *
     *
     * @param {ILexerConfig} [config=DEFAULT_LEXER_CONFIG] -
     *                  The Lexer's configuration @see {ILexerConfig} for details.
     */
    function Lexer(lexerDefinition, config) {
        if (config === void 0) { config = DEFAULT_LEXER_CONFIG; }
        var _this = this;
        this.lexerDefinition = lexerDefinition;
        this.lexerDefinitionErrors = [];
        this.modes = [];
        this.allPatterns = {};
        this.patternIdxToType = {};
        this.patternIdxToGroup = {};
        this.patternIdxToLongerAltIdx = {};
        this.patternIdxToCanLineTerminator = {};
        this.patternIdxToIsCustom = {};
        this.patternIdxToPushMode = {};
        this.patternIdxToPopMode = {};
        this.emptyGroups = {};
        this.config = undefined;
        this.trackStartLines = true;
        this.trackEndLines = true;
        if (typeof config === "boolean") {
            throw Error("The second argument to the Lexer constructor is now an ILexerConfig Object.\n" +
                "a boolean 2nd argument is no longer supported");
        }
        this.config = utils_1.merge(DEFAULT_LEXER_CONFIG, config);
        if (this.config.debug === true) {
            console.log("Running the Lexer in Debug Mode, DO NOT ENABLE THIS IN PRODUCTIVE ENV!");
        }
        this.trackStartLines = /full|onlyStart/i.test(this.config.positionTracking);
        this.trackEndLines = /full/i.test(this.config.positionTracking);
        var hasOnlySingleMode = true;
        var actualDefinition;
        // Convert SingleModeLexerDefinition into a IMultiModeLexerDefinition.
        if (utils_1.isArray(lexerDefinition)) {
            actualDefinition = { modes: {} };
            actualDefinition.modes[lexer_1.DEFAULT_MODE] = utils_1.cloneArr(lexerDefinition);
            actualDefinition[lexer_1.DEFAULT_MODE] = lexer_1.DEFAULT_MODE;
        }
        else {
            hasOnlySingleMode = false;
            actualDefinition = utils_1.cloneObj(lexerDefinition);
        }
        this.lexerDefinitionErrors = this.lexerDefinitionErrors.concat(lexer_1.performRuntimeChecks(actualDefinition));
        // for extra robustness to avoid throwing an none informative error message
        actualDefinition.modes = actualDefinition.modes ? actualDefinition.modes : {};
        // an error of undefined TokenClasses will be detected in "performRuntimeChecks" above.
        // this transformation is to increase robustness in the case of partially invalid lexer definition.
        utils_1.forEach(actualDefinition.modes, function (currModeValue, currModeName) {
            actualDefinition.modes[currModeName] = utils_1.reject(currModeValue, function (currTokClass) { return utils_1.isUndefined(currTokClass); });
        });
        var allModeNames = utils_1.keys(actualDefinition.modes);
        utils_1.forEach(actualDefinition.modes, function (currModDef, currModName) {
            _this.modes.push(currModName);
            _this.lexerDefinitionErrors = _this.lexerDefinitionErrors.concat(lexer_1.validatePatterns(currModDef, allModeNames));
            // If definition errors were encountered, the analysis phase may fail unexpectedly/
            // Considering a lexer with definition errors may never be used, there is no point
            // to performing the analysis anyhow...
            if (utils_1.isEmpty(_this.lexerDefinitionErrors)) {
                tokens_1.augmentTokenClasses(currModDef);
                var currAnalyzeResult = lexer_1.analyzeTokenClasses(currModDef);
                _this.allPatterns[currModName] = currAnalyzeResult.allPatterns;
                _this.patternIdxToType[currModName] = currAnalyzeResult.patternIdxToType;
                _this.patternIdxToGroup[currModName] = currAnalyzeResult.patternIdxToGroup;
                _this.patternIdxToLongerAltIdx[currModName] = currAnalyzeResult.patternIdxToLongerAltIdx;
                _this.patternIdxToCanLineTerminator[currModName] = currAnalyzeResult.patternIdxToCanLineTerminator;
                _this.patternIdxToIsCustom[currModName] = currAnalyzeResult.patternIdxToIsCustom;
                _this.patternIdxToPushMode[currModName] = currAnalyzeResult.patternIdxToPushMode;
                _this.patternIdxToPopMode[currModName] = currAnalyzeResult.patternIdxToPopMode;
                _this.emptyGroups = utils_1.merge(_this.emptyGroups, currAnalyzeResult.emptyGroups);
            }
        });
        this.defaultMode = actualDefinition.defaultMode;
        var allTokensTypes = utils_1.flatten(utils_1.mapValues(actualDefinition.modes, function (currModDef) { return currModDef; }));
        this.hasCustomTokens = lexer_1.checkHasCustomTokenPatterns(allTokensTypes);
        if (!utils_1.isEmpty(this.lexerDefinitionErrors) && !this.config.deferDefinitionErrorsHandling) {
            var allErrMessages = utils_1.map(this.lexerDefinitionErrors, function (error) {
                return error.message;
            });
            var allErrMessagesString = allErrMessages.join("-----------------------\n");
            throw new Error("Errors detected in definition of Lexer:\n" + allErrMessagesString);
        }
        if (lexer_1.SUPPORT_STICKY) {
            this.chopInput = utils_1.IDENTITY;
        }
        else {
            this.updateLastIndex = utils_1.NOOP;
        }
        if (hasOnlySingleMode) {
            this.handleModes = utils_1.NOOP;
        }
        if (!this.hasCustomTokens) {
            this.matchToPattern = this.matchPatternNoCustom;
        }
        if (this.trackStartLines === false) {
            this.computeNewColumn = utils_1.IDENTITY;
        }
        if (this.trackEndLines === false) {
            this.updateTokenEndLineColumnLocation = utils_1.NOOP;
        }
        if (/full/i.test(this.config.positionTracking)) {
            this.createTokenInstance = this.createFullToken;
        }
        else if (/onlyStart/i.test(this.config.positionTracking)) {
            this.createTokenInstance = this.createStartOnlyToken;
        }
        else if (/onlyOffset/i.test(this.config.positionTracking)) {
            this.createTokenInstance = this.createOffsetOnlyToken;
        }
        else {
            throw Error("Invalid <positionTracking> config option: \"" + this.config.positionTracking + "\"");
        }
    }
    /**
     * Will lex(Tokenize) a string.
     * Note that this can be called repeatedly on different strings as this method
     * does not modify the state of the Lexer.
     *
     * @param {string} text - The string to lex
     * @param {string} [initialMode] - The initial Lexer Mode to start with, by default this will be the first mode in the lexer's
     *                                 definition. If the lexer has no explicit modes it will be the implicit single 'default_mode' mode.
     *
     * @returns {ILexingResult}
     */
    Lexer.prototype.tokenize = function (text, initialMode) {
        if (initialMode === void 0) { initialMode = this.defaultMode; }
        if (!utils_1.isEmpty(this.lexerDefinitionErrors)) {
            var allErrMessages = utils_1.map(this.lexerDefinitionErrors, function (error) {
                return error.message;
            });
            var allErrMessagesString = allErrMessages.join("-----------------------\n");
            throw new Error("Unable to Tokenize because Errors detected in definition of Lexer:\n" + allErrMessagesString);
        }
        var lexResult = this.tokenizeInternal(text, initialMode);
        if (this.config.debug === true) {
            this.addTokenTypeNamesToResult(lexResult);
        }
        return lexResult;
    };
    // There is quite a bit of duplication between this and "tokenizeInternalLazy"
    // This is intentional due to performance considerations.
    Lexer.prototype.tokenizeInternal = function (text, initialMode) {
        var _this = this;
        var match, i, j, matchAlt, longerAltIdx, matchedImage, imageLength, group, tokType, newToken, errLength, droppedChar, lastLTIdx, msg;
        var orgText = text;
        var orgLength = orgText.length;
        var offset = 0;
        var matchedTokens = [];
        var errors = [];
        var line = this.trackStartLines ? 1 : undefined;
        var column = this.trackStartLines ? 1 : undefined;
        var groups = lexer_1.cloneEmptyGroups(this.emptyGroups);
        var trackLines = this.trackStartLines;
        var currModePatterns = [];
        var currModePatternsLength = 0;
        var currModePatternIdxToLongerAltIdx = [];
        var currModePatternIdxToIsCustom = [];
        var currModePatternIdxToGroup = [];
        var currModePatternIdxToType = [];
        var currModePatternIdxToCanLineTerminator = [];
        var patternIdxToPushMode = [];
        var patternIdxToPopMode = [];
        var modeStack = [];
        var pop_mode = function (popToken) {
            // TODO: perhaps avoid this error in the edge case there is no more input?
            if (modeStack.length === 1) {
                // if we try to pop the last mode there lexer will no longer have ANY mode.
                // thus the pop is ignored, an error will be created and the lexer will continue parsing in the previous mode.
                var msg_1 = "Unable to pop Lexer Mode after encountering Token ->" + popToken.image + "<- The Mode Stack is empty";
                errors.push({
                    offset: popToken.startOffset,
                    line: popToken.startLine !== undefined ? popToken.startLine : undefined,
                    column: popToken.startColumn !== undefined ? popToken.startColumn : undefined,
                    length: popToken.image.length,
                    message: msg_1
                });
            }
            else {
                modeStack.pop();
                var newMode = utils_1.last(modeStack);
                currModePatterns = _this.allPatterns[newMode];
                currModePatternsLength = currModePatterns.length;
                currModePatternIdxToLongerAltIdx = _this.patternIdxToLongerAltIdx[newMode];
                currModePatternIdxToIsCustom = _this.patternIdxToIsCustom[newMode];
                currModePatternIdxToGroup = _this.patternIdxToGroup[newMode];
                currModePatternIdxToType = _this.patternIdxToType[newMode];
                currModePatternIdxToCanLineTerminator = _this.patternIdxToCanLineTerminator[newMode];
                patternIdxToPushMode = _this.patternIdxToPushMode[newMode];
                patternIdxToPopMode = _this.patternIdxToPopMode[newMode];
            }
        };
        function push_mode(newMode) {
            modeStack.push(newMode);
            currModePatterns = this.allPatterns[newMode];
            currModePatternsLength = currModePatterns.length;
            currModePatternIdxToLongerAltIdx = this.patternIdxToLongerAltIdx[newMode];
            currModePatternIdxToIsCustom = this.patternIdxToIsCustom[newMode];
            currModePatternIdxToGroup = this.patternIdxToGroup[newMode];
            currModePatternIdxToType = this.patternIdxToType[newMode];
            currModePatternIdxToCanLineTerminator = this.patternIdxToCanLineTerminator[newMode];
            patternIdxToPushMode = this.patternIdxToPushMode[newMode];
            patternIdxToPopMode = this.patternIdxToPopMode[newMode];
        }
        // this pattern seems to avoid a V8 de-optimization, although that de-optimization does not
        // seem to matter performance wise.
        push_mode.call(this, initialMode);
        while (offset < orgLength) {
            match = null;
            for (i = 0; i < currModePatternsLength; i++) {
                var currPattern = currModePatterns[i];
                match = this.matchToPattern(currPattern, offset, text, currModePatternIdxToIsCustom, i, orgText, matchedTokens, groups);
                if (match !== null) {
                    // even though this pattern matched we must try a another longer alternative.
                    // this can be used to prioritize keywords over identifiers
                    longerAltIdx = currModePatternIdxToLongerAltIdx[i];
                    if (longerAltIdx !== undefined) {
                        var longerAltPattern = currModePatterns[longerAltIdx];
                        matchAlt = this.matchToPattern(longerAltPattern, offset, text, currModePatternIdxToIsCustom, i, orgText, matchedTokens, groups);
                        if (matchAlt && matchAlt[0].length > match[0].length) {
                            match = matchAlt;
                            i = longerAltIdx;
                        }
                    }
                    break;
                }
            }
            // successful match
            if (match !== null) {
                matchedImage = match[0];
                imageLength = matchedImage.length;
                group = currModePatternIdxToGroup[i];
                if (group !== undefined) {
                    tokType = currModePatternIdxToType[i];
                    // TODO: "offset + imageLength" and the new column may be computed twice in case of "full" location information inside
                    // createFullToken method
                    newToken = this.createTokenInstance(matchedImage, offset, tokType, line, column, imageLength);
                    if (group === false) {
                        matchedTokens.push(newToken);
                    }
                    else {
                        groups[group].push(newToken);
                    }
                }
                text = this.chopInput(text, imageLength);
                offset = offset + imageLength;
                // TODO: with newlines the column may be assigned twice
                column = this.computeNewColumn(column, imageLength);
                if (trackLines === true && currModePatternIdxToCanLineTerminator[i] === true) {
                    var numOfLTsInMatch = 0;
                    var imageOffset = 0;
                    var imageLengthMinusOne = imageLength - 1;
                    while (imageOffset < matchedImage.length) {
                        var c = matchedImage.charCodeAt(imageOffset);
                        if (c === 10) {
                            numOfLTsInMatch++;
                            lastLTIdx = imageOffset;
                        }
                        else if (c === 13) {
                            if (imageOffset !== imageLengthMinusOne &&
                                matchedImage.charCodeAt(imageOffset + 1) === 10) {
                            }
                            else {
                                numOfLTsInMatch++;
                                lastLTIdx = imageOffset;
                            }
                        }
                        imageOffset++;
                    }
                    if (numOfLTsInMatch !== 0) {
                        line = line + numOfLTsInMatch;
                        column = imageLength - lastLTIdx;
                        this.updateTokenEndLineColumnLocation(newToken, group, lastLTIdx, numOfLTsInMatch, line, column, imageLength);
                    }
                }
                // will be NOOP if no modes present
                this.handleModes(i, patternIdxToPopMode, patternIdxToPushMode, pop_mode, push_mode, newToken);
            }
            else {
                var errorStartOffset = offset;
                var errorLine = line;
                var errorColumn = column;
                var foundResyncPoint = false;
                while (!foundResyncPoint && offset < orgLength) {
                    // drop chars until we succeed in matching something
                    droppedChar = orgText.charCodeAt(offset);
                    if (droppedChar === 10 ||
                        (droppedChar === 13 &&
                            (offset === orgLength - 1 ||
                                (offset < orgLength - 1 && orgText.charCodeAt(offset + 1) !== 10)))) {
                        // followed by
                        // '\n'
                        line++;
                        column = 1;
                    }
                    else {
                        // either when skipping the next char, or when consuming the following pattern
                        // (which will have to start in a '\n' if we manage to consume it)
                        column++;
                    }
                    // Identity Func (when sticky flag is enabled)
                    text = this.chopInput(text, 1);
                    offset++;
                    for (j = 0; j < currModePatterns.length; j++) {
                        var currPattern = currModePatterns[j];
                        foundResyncPoint = this.matchToPattern(currPattern, offset, text, currModePatternIdxToIsCustom, j, orgText, matchedTokens, groups);
                        if (foundResyncPoint !== null) {
                            break;
                        }
                    }
                }
                errLength = offset - errorStartOffset;
                // at this point we either re-synced or reached the end of the input text
                msg = "unexpected character: ->" + orgText.charAt(errorStartOffset) + "<- at offset: " + errorStartOffset + "," +
                    (" skipped " + (offset - errorStartOffset) + " characters.");
                errors.push({ offset: errorStartOffset, line: errorLine, column: errorColumn, length: errLength, message: msg });
            }
        }
        return { tokens: matchedTokens, groups: groups, errors: errors };
    };
    Lexer.prototype.handleModes = function (i, patternIdxToPopMode, patternIdxToPushMode, pop_mode, push_mode, newToken) {
        if (patternIdxToPopMode[i] === true) {
            // need to save the PUSH_MODE property as if the mode is popped
            // patternIdxToPopMode is updated to reflect the new mode after popping the stack
            var pushMode = patternIdxToPushMode[i];
            pop_mode(newToken);
            if (pushMode !== undefined) {
                push_mode.call(this, pushMode);
            }
        }
        else if (patternIdxToPushMode[i] !== undefined) {
            push_mode.call(this, patternIdxToPushMode[i]);
        }
    };
    Lexer.prototype.matchToPattern = function (currPattern, offset, text, currModePatternIdxToIsCustom, i, orgText, matchedTokens, groups) {
        if (currModePatternIdxToIsCustom[i] === true) {
            return currPattern.exec(orgText, offset, matchedTokens, groups);
        }
        else {
            this.updateLastIndex(currPattern, offset);
            return currPattern.exec(text);
        }
    };
    Lexer.prototype.matchPatternNoCustom = function (currPattern, offset, text) {
        this.updateLastIndex(currPattern, offset);
        return currPattern.exec(text);
    };
    Lexer.prototype.chopInput = function (text, length) {
        return text.substring(length);
    };
    Lexer.prototype.updateLastIndex = function (regExp, newLastIndex) {
        regExp.lastIndex = newLastIndex;
    };
    // TODO: decrease this under 600 characters? inspect stripping comments option in TSC compiler
    Lexer.prototype.updateTokenEndLineColumnLocation = function (newToken, group, lastLTIdx, numOfLTsInMatch, line, column, imageLength) {
        var lastCharIsLT, fixForEndingInLT;
        if (group !== undefined) {
            lastCharIsLT = lastLTIdx === imageLength - 1;
            fixForEndingInLT = lastCharIsLT ?
                -1 :
                0;
            if (!(numOfLTsInMatch === 1 && lastCharIsLT === true)) {
                // if a token ends in a LT that last LT only affects the line numbering of following Tokens
                newToken.endLine = line + fixForEndingInLT;
                // the last LT in a token does not affect the endColumn either as the [columnStart ... columnEnd)
                // inclusive to exclusive range.
                newToken.endColumn = column - 1 + -fixForEndingInLT;
            }
            // else single LT in the last character of a token, no need to modify the endLine/EndColumn
        }
    };
    Lexer.prototype.computeNewColumn = function (oldColumn, imageLength) {
        return oldColumn + imageLength;
    };
    // Place holder, will be replaced by the correct variant according to the locationTracking option at runtime.
    /* istanbul ignore next - place holder */
    Lexer.prototype.createTokenInstance = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return null;
    };
    Lexer.prototype.createOffsetOnlyToken = function (image, startOffset, tokenType) {
        return {
            image: image,
            startOffset: startOffset,
            tokenType: tokenType
        };
    };
    Lexer.prototype.createStartOnlyToken = function (image, startOffset, tokenType, startLine, startColumn) {
        return {
            image: image,
            startOffset: startOffset,
            startLine: startLine,
            startColumn: startColumn,
            tokenType: tokenType
        };
    };
    Lexer.prototype.createFullToken = function (image, startOffset, tokenType, startLine, startColumn, imageLength) {
        return {
            image: image,
            startOffset: startOffset,
            endOffset: startOffset + imageLength - 1,
            startLine: startLine,
            endLine: startLine,
            startColumn: startColumn,
            endColumn: startColumn + imageLength - 1,
            tokenType: tokenType
        };
    };
    Lexer.prototype.addTokenTypeNamesToResult = function (lexResult) {
        utils_1.forEach(lexResult.tokens, function (currToken) {
            currToken.tokenClassName = tokens_public_1.tokenName(tokens_public_1.getTokenConstructor(currToken));
        });
        utils_1.forEach(lexResult.groups, function (currGroup) {
            utils_1.forEach(currGroup, function (currToken) {
                currToken.tokenClassName = tokens_public_1.tokenName(tokens_public_1.getTokenConstructor(currToken));
            });
        });
    };
    return Lexer;
}());
Lexer.SKIPPED = "This marks a skipped Token pattern, this means each token identified by it will" +
    "be consumed and then thrown into oblivion, this can be used to for example to completely ignore whitespace.";
Lexer.NA = /NOT_APPLICABLE/;
exports.Lexer = Lexer;
//# sourceMappingURL=lexer_public.js.map