"use strict";
var __extends = (this && this.__extends) || (function () {
    /* istanbul ignore next */ 
var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) /* istanbul ignore next */  if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        /* istanbul ignore next */ 
d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils/utils");
var lang_extensions_1 = require("../lang/lang_extensions");
var lexer_public_1 = require("./lexer_public");
var tokens_1 = require("./tokens");
/**
 *  This can be used to improve the quality/readability of error messages or syntax diagrams.
 *
 * @param {Function} clazz - A constructor for a Token subclass
 * @returns {string} - The Human readable label for a Token if it exists.
 */
function tokenLabel(clazz) {
    if (hasTokenLabel(clazz)) {
        return clazz.LABEL;
    }
    else {
        return tokenName(clazz);
    }
}
exports.tokenLabel = tokenLabel;
function hasTokenLabel(clazz) {
    return utils_1.isString(clazz.LABEL) && clazz.LABEL !== "";
}
exports.hasTokenLabel = hasTokenLabel;
function tokenName(clazz) {
    // The tokenName property is needed under some old versions of node.js (0.10/0.12)
    // where the Function.prototype.name property is not defined as a 'configurable' property
    // enable producing readable error messages.
    /* istanbul ignore if -> will only run in old versions of node.js */
    if (utils_1.isObject(clazz) && clazz.hasOwnProperty("tokenName") && utils_1.isString(clazz.tokenName)) {
        return clazz.tokenName;
    }
    else {
        return lang_extensions_1.functionName(clazz);
    }
}
exports.tokenName = tokenName;
var PARENT = "parent";
var LABEL = "label";
var GROUP = "group";
var PUSH_MODE = "push_mode";
var POP_MODE = "pop_mode";
var LONGER_ALT = "longer_alt";
/**
 * @param {ITokenConfig} config - The configuration for
 * @returns {TokenConstructor} - A constructor for the new Token subclass
 */
function createToken(config) {
    if (!utils_1.has(config, PARENT)) {
        config.parent = Token;
    }
    return createTokenInternal(config);
}
exports.createToken = createToken;
/**
 *
 * @deprecated - Use the new CreateToken API
 *
 * utility to help the poor souls who are still stuck writing pure javascript 5.1
 * extend and create Token subclasses in a less verbose manner
 *
 * @param {string} tokenName - The name of the new TokenClass
 * @param {RegExp|CustomPatternMatcherFunc|Function} patternOrParent - RegExp Pattern or Parent Token Constructor
 * @param {Function} parentConstructor - The Token class to be extended
 * @returns {Function} - A constructor for the new extended Token subclass
 */
function extendToken(tokenName, patternOrParent, parentConstructor) {
    if (patternOrParent === void 0) { patternOrParent = undefined; }
    if (parentConstructor === void 0) { parentConstructor = Token; }
    var pattern;
    if (utils_1.isRegExp(patternOrParent) ||
        patternOrParent === lexer_public_1.Lexer.SKIPPED ||
        patternOrParent === lexer_public_1.Lexer.NA) {
        pattern = patternOrParent;
    }
    else if (utils_1.isFunction(patternOrParent)) {
        parentConstructor = patternOrParent;
        pattern = undefined;
    }
    return createTokenInternal({ name: tokenName, parent: parentConstructor, pattern: pattern });
}
exports.extendToken = extendToken;
function createTokenInternal(config) {
    var tokenName = config.name;
    var parentConstructor = config.parent;
    var pattern = config.pattern;
    var derivedConstructor = function () {
        parentConstructor.apply(this, arguments);
    };
    // can be overwritten according to:
    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/
    // name?redirectlocale=en-US&redirectslug=JavaScript%2FReference%2FGlobal_Objects%2FFunction%2Fname
    /* istanbul ignore if -> will only run in old versions of node.js */
    if (!lang_extensions_1.defineNameProp(derivedConstructor, tokenName)) {
        // hack to save the tokenName in situations where the constructor's name property cannot be reconfigured
        derivedConstructor.tokenName = tokenName;
    }
    derivedConstructor.prototype = Object.create(parentConstructor.prototype);
    derivedConstructor.prototype.constructor = derivedConstructor;
    if (!utils_1.isUndefined(pattern)) {
        derivedConstructor.PATTERN = pattern;
    }
    tokens_1.augmentTokenClasses([derivedConstructor]);
    // static properties mixing
    derivedConstructor = utils_1.assignNoOverwrite(derivedConstructor, parentConstructor);
    if (utils_1.has(config, LABEL)) {
        derivedConstructor.LABEL = config[LABEL];
    }
    if (utils_1.has(config, GROUP)) {
        derivedConstructor.GROUP = config[GROUP];
    }
    if (utils_1.has(config, POP_MODE)) {
        derivedConstructor.POP_MODE = config[POP_MODE];
    }
    if (utils_1.has(config, PUSH_MODE)) {
        derivedConstructor.PUSH_MODE = config[PUSH_MODE];
    }
    if (utils_1.has(config, LONGER_ALT)) {
        derivedConstructor.LONGER_ALT = config[LONGER_ALT];
    }
    return derivedConstructor;
}
var Token = (function () {
    /**
     * This class is never meant to be initialized.
     * The class hierarchy is used to organize Token metadata, not to create instances of Tokens.
     * Tokens are simple JavaScript objects which are NOT created using the <new> operator.
     * To get the class of a Token "instance" use <getTokenConstructor>.
     */
    function Token() {
        // this marks if a Token does not really exist and has been inserted "artificially" during parsing in rule error recovery
        this.isInsertedInRecovery = false;
    }
    return Token;
}());
/**
 * A "human readable" Label for a Token.
 * Subclasses of Token may define their own static LABEL property.
 * This label will be used in error messages and drawing syntax diagrams.
 *
 * For example a Token constructor may be called LCurly, which is short for LeftCurlyBrackets, These names are either too short
 * or too unwieldy to be used in error messages.
 *
 * Imagine : "expecting LCurly but found ')'" or "expecting LeftCurlyBrackets but found ')'"
 *
 * However if a static property LABEL with the value '{' exists on LCurly class, that error message will be:
 * "expecting '{' but found ')'"
 */
Token.LABEL = undefined;
exports.Token = Token;
var EOF = (function (_super) {
    __extends(EOF, _super);
    function EOF() {
        return _super !== null && _super.apply(this, arguments) /* istanbul ignore next */ || this;
    }
    return EOF;
}(Token));
exports.EOF = EOF;
tokens_1.augmentTokenClasses([EOF]);
/**
 * Utility to create Chevrotain Token "instances"
 * Note that Chevrotain tokens are not real instances, and thus the instanceOf cannot be used.
 *
 * @param tokClass
 * @param image
 * @param startOffset
 * @param endOffset
 * @param startLine
 * @param endLine
 * @param startColumn
 * @param endColumn
 * @returns {{image: string,
 *            startOffset: number,
 *            endOffset: number,
 *            startLine: number,
 *            endLine: number,
 *            startColumn: number,
 *            endColumn: number,
 *            tokenType}}
 */
function createTokenInstance(tokClass, image, startOffset, endOffset, startLine, endLine, startColumn, endColumn) {
    return {
        image: image,
        startOffset: startOffset,
        endOffset: endOffset,
        startLine: startLine,
        endLine: endLine,
        startColumn: startColumn,
        endColumn: endColumn,
        tokenType: tokClass.tokenType,
    };
}
exports.createTokenInstance = createTokenInstance;
/**
 * Given a Token instance, will return the Token Constructor.
 * Note that this function is not just for convenience, Because a Token "instance'
 * Does not use standard prototype inheritance and thus it's constructor cannot be accessed
 * by traversing the prototype chain.
 *
 * @param tokenInstance {IToken}
 * @returns {TokenConstructor}
 */
function getTokenConstructor(tokenInstance) {
    var tokenIdx;
    tokenIdx = tokenInstance.tokenType;
    return tokens_1.tokenIdxToClass.get(tokenIdx);
}
exports.getTokenConstructor = getTokenConstructor;
/**
 * A Utility method to check if a token is of the type of the argument Token class.
 * Not that while this utility has similar semantics to ECMAScript "instanceOf"
 * As Chevrotain tokens support inheritance.
 *
 * It is not actually implemented using the "instanceOf" operator because
 * Chevrotain Tokens have their own performance optimized inheritance mechanism.
 *
 * @param tokInstance {IToken}
 * @param tokClass {TokenConstructor}
 * @returns {boolean}
 */
function tokenMatcher(tokInstance, tokClass) {
    return tokens_1.tokenStructuredMatcher(tokInstance, tokClass);
}
exports.tokenMatcher = tokenMatcher;
//# sourceMappingURL=tokens_public.js.map